# -*- Mode: Python; tab-width: 4 -*-

import string

import mvc
import mvc_scrollbar
import winwin
import winfont
import mvc_seq_win

user32 = winwin.user32

SPLIT_WORD = winwin.SPLIT_WORD

# TODO: collapse seq_win and cell_win more completely, rather
# than duplicating all this code. (I think seq_win ought to use
# get_cell_size(), too).
#
# A good demo for this would be a program that displayed a PBM
# file using big fat square pixels, and then let the user edit
# it by clicking the pixels on and off...

# sequence of sequences?

class cell_window (mvc_seq_win.sequence_window):
	
	def get_position (self):
		vm, hm = self.scroll_models
		cx, cy = self.get_cell_size()
		return cx * hm.position, cy * vm.position

	def get_limits (self):
		return (
			max (map (len, self.model)),
			len (self.model)
			)

	def WM_SIZE (self, wparam, lparam):
		self.size = SPLIT_WORD (lparam)
		w, h = self.size
		cx, cy = self.get_cell_size()
		vm, hm = self.scroll_models
		hm.limit, vm.limit = self.get_limits()
		hm.page = w / cx
		vm.page = h / cy

	def draw_line (self, dc, index):
		# sample implementation
		cx, cy = self.get_cell_size()
		y = cy * index
		dc.select_object (self.font)
		dc.text_out ((0,y), str(self.model[index]))
			
	def notify (self, model, hint):
		vm, hm = self.scroll_models

		# a change to the sequence model
		if model is self.model:
			kind = hint[0]
			if kind == model.HINT_CHANGE:
				ignore, low, high = hint
				self.invalidate_lines (low, high)
			elif kind == model.HINT_INSERT:
				ignore, start, nitems = hint
				self.invalidate_lines (start, len(self.model))
				vm.limit = len(self.model)
			elif kind == model.HINT_REMOVE:
				ignore, start, nitems = hint
				self.invalidate_lines (start, len(self.model))
				vm.limit = len(self.model)

		# a scroll event
		elif model in self.scroll_models:
			lx, ly = self.last_scroll_position
			nx, ny = self.get_position()

			if model is vm:
				dx, dy = 0, ly - ny
			elif model is hm:
				dx, dy = lx - nx, 0

			self.scroll_window (dx, dy)
			self.last_scroll_position = nx, ny
			self.update_window()

class text_window (cell_window):

	font_height = 16
	font = winfont.font ('Courier New', height=font_height).create()
	cell_size = None

	def get_cell_size (self):
		if self.cell_size is None:
			# don't call ours, it needs cell_size!
			dc = winwin.python_window.get_dc (self)
			dc.select_object (self.font)
			tm = dc.get_text_metrics()
			dc.release_dc()
			self.cell_size = tm.ave_char_width, tm.height

		return self.cell_size

class text_edit_window (text_window):
	
	cursor_pos = (0,0)

	def WM_SETFOCUS (self, wparam, lparam):
		self.show_caret()
	
	def show_caret (self):
		cx, cy = self.get_cell_size()
		user32.CreateCaret (self, 0, cx, cy)
		self.set_caret_pos()
		user32.ShowCaret (self)

	def set_caret_pos (self):
		cx, cy = self.get_cell_size()
		x, y = self.cursor_pos
		x, y = self.lp_to_dp ((x * cx, y * cy))
		user32.SetCaretPos (x, y)

	def WM_KILLFOCUS (self, wparam, lparam):
		user32.DestroyCaret (self)

	def WM_LBUTTONDOWN (self, wparam, lparam):
		print 'lbuttondown', SPLIT_WORD (lparam)
		x, y = self.dp_to_lp (SPLIT_WORD (lparam))
		cx, cy = self.get_cell_size()
		self.cursor_pos = x/cx, y/cy
		self.set_caret_pos ()

	def WM_CHAR (self, wparam, lparam):
		cx, cy = self.get_cell_size()
		ch = chr (wparam)
		w, h = self.size
		x, y = self.cursor_pos

		line = self.model[y]

		if ch == '\r':
			if x < len(line):
				self.model[y] = line[:x]
				self.model.insert (y+1, line[x:])
			else:
				self.model.insert (y+1, '')
			self.cursor_pos = 0, y+1
			#self.invalidate_rect ((0,y * lh,w,h))
		else:
			chars = map (None, line)
			chars.insert (x, ch)
			self.model[y] = string.join (chars, '')
			x = x + 1
			self.cursor_pos = x, y

		self.set_caret_pos()

		return 1

class text_component (mvc_seq_win.sequence_component):
 
	sequence_view_class = text_window

class text_edit_component (mvc_seq_win.sequence_component):
 
	def WM_SETFOCUS (self, wparam, lparam):
		self.sequence_window.set_focus()

	sequence_view_class = text_edit_window


def demo():
	
	def frob_line (line):
		import string

		line = string.join (string.split (line, '\t'), '    ')
		while line and line[-1] in '\r\n':
			line = line[:-1]
		return line
		
	lines = open ('mvc_cell_win.py', 'rb').readlines()
	lines = map (frob_line, lines)
	lines = mvc.sequence_model (lines)
	
	w = text_edit_component (
		'demo cell window',
		style= winwin.WS_OVERLAPPEDWINDOW
		).create()

	w.set_model (lines)
	w.show_window()
	return w

if __name__ == '__main__':
	import msgloop
	w = demo()
	msgloop.go()
